import gevent

#https://www.snip2code.com/Snippet/506288/Gevent-based-Tail-F-generator


class GeventTail():
    def __init__(self, *args, **kwargs):
        self.file_name = kwargs.pop('file_name')
        try:
            self.fd = os.open(self.file_name, os.O_RDONLY | os.O_NONBLOCK)
            os.lseek(self.fd, 0, os.SEEK_END)
        except:
            self.fd = None
        self.hub = gevent.get_hub()
        self.watcher = self.hub.loop.stat(self.file_name)

    def readline(self):
        while self.fd:
            lines = os.read(self.fd, 4096).splitlines()
            if lines:
                for line in lines:
                    yield line
            else:
                self.hub.wait(self.watcher)



import os
import re
import gevent
import requests
import signal
import requests
import apache_log_parser
import argparse

from gevent.queue import Queue
from gevent import monkey

from .tail import GeventTail

monkey.patch_socket()

DEFAULT_LOG_FORMAT = "%h %l %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-Agent}i\""


def match_keywords(keywords, request_url):
    for k in keywords:
        if k in request_url:
            return True
    return False


def worker(args, line, line_parser):
    l = line_parser(line)

    url = '%s%s' % (args.server.rstrip('/'), l['request_url'])

    match = args.match is None or match_keywords(args.match, l['request_url'])
    ignore = args.ignore is not None and match_keywords(args.ignore, l['request_url'])

    if match and not ignore:
        print url
        if not args.dry_run and match:
            r = requests.get(url, auth=args.auth)
    else:
        print '[ignored] %s' % (url)


def reader(args):
    line_parser = apache_log_parser.make_parser(args.format)

    if args.auth is not None:
        credentials = args.auth.split(':')
        args.auth = requests.auth.HTTPBasicAuth(credentials[0], credentials[1])

    gt = GeventTail(file_name=args.log_file)
    for line in gt.readline():
        gevent.spawn(worker, args, line, line_parser).join()


def main():
    gevent.signal(signal.SIGQUIT, gevent.kill)

    parser = argparse.ArgumentParser(description='Apache Log Replicator')

    parser.add_argument('-a', '--auth', help='Basic authentication user:password', type=str)

    parser.add_argument('-m', '--match', help='Only process matching requests', type=str)

    parser.add_argument('-i', '--ignore', help='Ignore matching requests', type=str)

    parser.add_argument('-d', '--dry-run', dest='dry_run', action='store_true')

    parser.add_argument('-f', '--format', help='Apache log format', type=str, default=DEFAULT_LOG_FORMAT)

    parser.add_argument('server', help='Remote Server')

    parser.add_argument('log_file', help='Apache log file path')

    args = parser.parse_args()

    if args.match is not None:
        args.match = args.match.split('|')

    if args.ignore is not None:
        args.ignore = args.ignore.split('|')

    try:
        gevent.spawn(reader, args).join()
    except KeyboardInterrupt:
        pass


if __name__ == '__main__':
    main()



import sys

from setuptools import setup
from areplay import __version__

install_requires = ['apache-log-parser', 'gevent', 'requests', ]

if sys.version_info < (2, 7):
    install_requires += ['argparse']

classifiers = [
    "Programming Language :: Python",
    "License :: OSI Approved :: MIT License",
    "Development Status :: 1 - Planning"
]

description = ''

for file_ in ('README', 'CHANGES', 'CONTRIBUTORS'):
    with open('%s.rst' % file_) as f:
        description += f.read() + '\n\n'

setup(
    name='areplay',
    packages=['areplay'],
    version=__version__,
    description='Apache Log live replicator',
    author='Hugo Dias',
    author_email='hdias@synchlabs.com',
    url='https://github.com/ssynchron/areplay',
    download_url='https://github.com/ssynchron/areplay/tarball/' + __version__,
    keywords=['testing', 'logging'],
    include_package_data=True,
    classifiers=classifiers,
    install_requires=install_requires,
    entry_points={
        'console_scripts': [
            'areplay=areplay.main:main',
        ],
    }
)

